/**
 * 
 */
package com.axemble.education.connectors.smtp;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Date;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.w3c.dom.Element;

import com.axemble.education.connectors.smtp.SampleSMTPConnection.InnerConnection;
import com.axemble.vdoc.sdk.connectors.BaseTreatmentConnector;
import com.axemble.vdoc.sdk.document.fields.LocalizedTextField;
import com.axemble.vdoc.sdk.exceptions.SDKException;
import com.axemble.vdoc.sdk.interfaces.IContext;
import com.axemble.vdoc.sdk.interfaces.ITaskInstance;
import com.axemble.vdoc.sdk.interfaces.IUser;
import com.axemble.vdoc.sdk.interfaces.IWorkflowInstance;

public class SampleSendMailTreatmentConnector extends BaseTreatmentConnector
{
	private static final long serialVersionUID = -519594534740139407L;

	private byte[] messageBytes;

	/* (non-Javadoc)
	 * @see com.axemble.vdoc.sdk.connectors.BaseTreatmentConnector#execute(com.axemble.vdoc.sdk.interfaces.IContext, com.axemble.vdoc.sdk.interfaces.IWorkflowInstance, com.axemble.vdoc.sdk.interfaces.ITaskInstance, org.w3c.dom.Element)
	 */
	@Override
	public int execute( IContext context, IWorkflowInstance workflowInstance, ITaskInstance taskInstance, Element customElement )
	{
		if ( context == null )
			context = getWorkflowModule().getContextByLogin( "sysadmin" );

		// Use another connection
		InnerConnection innerConnection = null;
		try
		{
			innerConnection = (InnerConnection)getConnection();

			// create the MimeMessage
			MimeMessage message = new MimeMessage( innerConnection.getSession() );

			// set the sender from the context
			InternetAddress senderAddress = new InternetAddress( context.getUser().getEmail(), context.getUser().getFullName() );
			message.setFrom( senderAddress );

			// set the recipients
			ArrayList<InternetAddress> toAddresses = new ArrayList<InternetAddress>();

			IUser user = getDirectoryModule().getUserByLogin( "sboirol" );
			toAddresses.add( new InternetAddress( user.getEmail(), user.getFullName() ) );

			InternetAddress[] toInternetAddresses = new InternetAddress[toAddresses.size()];
			toAddresses.toArray( toInternetAddresses );

			message.setRecipients( Message.RecipientType.TO, toInternetAddresses );

			// set the subject
			String subjectValue = (String)getTreatmentDefinition().getValue( SampleSendMailTreatmentConnectorEditor.FIELD_SUBJECT );
			if ( subjectValue.startsWith( "false;" ) )
			{
				subjectValue = LocalizedTextField.getValue( subjectValue );

				subjectValue = evaluateExpressions( context, workflowInstance, subjectValue );
			}
			else subjectValue = getStaticString( LocalizedTextField.getValue( subjectValue ) );

			message.setSubject( subjectValue );

			// set the body
			Object bodyContentValue = getTreatmentDefinition().getValue( SampleSendMailTreatmentConnectorEditor.FIELD_BODY );
			if ( bodyContentValue == null )
				throw new SDKException( getStaticString( "treatment.mail.sendMail.errors.emptyMessageContent" ) );

			String textBodyContentValue = new String( (byte[])bodyContentValue ).toString();

			textBodyContentValue = evaluateExpressions( context, workflowInstance, textBodyContentValue );

			message.setContent( textBodyContentValue, "text/html; charset=utf-8" );

			message.setSentDate( new Date() );

			// serialize the message for asynchronous treatment (see asyncExecute)
			serializeMessage( message );

			return IExecuteStatus.CONTINUE;
		}
		catch( Exception e )
		{
			return IExecuteStatus.STOP;
		}
		finally
		{
			safeClose( innerConnection );
		}
	}
	@Override
	public void beforeCompletion( IContext context, Element customElement )
	{
		super.beforeCompletion( context, customElement );
	}
	@Override
	public void afterCompletion( IContext context, Element customElement, boolean committed )
	{
		super.afterCompletion( context, customElement, committed );
	}
	@Override
	public void asyncExecute( Element customElement )
	{
		try
		{
			// Use another connection
			InnerConnection innerConnection = null;
			try
			{
				innerConnection = (InnerConnection)getConnection();

				// load the MimeMessage
				MimeMessage message = deserializeMessage( innerConnection.getSession(), this.messageBytes );

				// send mail using java mail API
				innerConnection.getTransport().sendMessage( message, message.getAllRecipients() );
			}
			finally
			{
				safeClose( innerConnection );
			}
		}
		catch( Exception e )
		{
			throw new SDKException( e );
		}
	}
	private String evaluateExpressions( IContext context, IWorkflowInstance workflowInstance, String text )
	{
		return text;
	}
	private void safeClose( InnerConnection innerConnection )
	{
		try
		{
			if ( innerConnection.getTransport() != null )
				innerConnection.getTransport().close();
		}
		catch( MessagingException e )
		{}
	}
	private void serializeMessage( MimeMessage message )
	{
		try
		{
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			message.writeTo( baos );
			this.messageBytes = baos.toByteArray();
		}
		catch( Exception e )
		{
			throw new SDKException( e );
		}
	}
	private MimeMessage deserializeMessage( Session session, byte[] bytes )
	{
		try
		{
			return new MimeMessage( session, new ByteArrayInputStream( bytes ) );
		}
		catch( Exception e )
		{
			throw new SDKException( e );
		}
	}

}
